/*	$NetBSD$	*/

/*-
 * Copyright (c) 2009 SHIMIZU Ryo <ryo@nerv.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/cdefs.h>
__KERNEL_RCSID(0, "$NetBSD$");

#include <sys/param.h>
#include <sys/bus.h>
#include <sys/device.h>

#include <arm/star/starreg.h>
#include <arm/star/starvar.h>

#include "locators.h"

struct star_softc {
	device_t sc_dev;

	bus_space_tag_t sc_iot;
	bus_space_tag_t sc_a4x_iot;

	bus_dma_tag_t sc_dmat;
};

int star_cpu_type;



static int star_match(device_t, struct cfdata *, void *);
static void star_attach(device_t, device_t, void *);
static int star_search(device_t, struct cfdata *, const int *, void *);
static int star_critical_search(device_t, struct cfdata *, const int *, void *);
static int star_search(device_t, struct cfdata *, const int *, void *);
static int star_print(void *, const char *);

CFATTACH_DECL_NEW(star, sizeof(struct star_softc),
    star_match, star_attach, NULL, NULL);

struct star_softc *star_sc = NULL;


/* ARGSUSED */
static int
star_match(device_t parent, struct cfdata *match, void *aux)
{
	return 1;
}

/* ARGSUSED */
static void
star_attach(device_t parent, device_t self, void *aux)
{
	struct star_softc *sc;
	struct star_attach_args sa;
	uint32_t id;
	int cpuclk;
	const char *cpu_family;

	sc = device_private(self);
	sc->sc_iot = &star_bs_tag;
	sc->sc_a4x_iot = &star_a4x_bs_tag;
	sc->sc_dmat = &star_bus_dma_tag;

	id = STAR_REG_READ32(STAR_MISC_PCI_ID_CAPABILITY);
	if (id == 0x8131eeee) {
		/* STR8100 */
		star_cpu_type = 8;
		cpu_family = "Star Equuleus Family SoC: STR8100";

		/* cpuclk: 0:175MHz, 1:200MHz, 2:225MHz, 3:250MHz */
		cpuclk = STAR_REG_READ32(STAR_CLKPWR_CLKCTRL_REG) &
		    STAR_CLKPWR_CLKCTRL_CPU_PLL_SEL_MASK;
#ifndef STAR_EQUULEUS
		panic("No support for STR8100 (STAR_EQUULEUS) in kernel");
#endif
	} else {
		/* STR9100? */
		star_cpu_type = 9;
		cpu_family = "Star Orion Family SoC: STR9100";
		cpuclk = -1;
#ifndef STAR_ORION
		panic("No support for STR9100 (STAR_ORION) in kernel");
#endif
	}

	aprint_normal(": %s", cpu_family);
	if (cpuclk < 0)
		aprint_normal("\n");
	else
		aprint_normal(", %dMHz\n", 175 + cpuclk * 25);
	aprint_naive("\n");

	config_search_ia(star_critical_search, self, "star", &sa);
	config_search_ia(star_search, self, "star", &sa);
}

/* ARGSUSED */
static int
star_critical_search(device_t parent, struct cfdata *cf, const int *ldesc,
    void *aux)
{
	struct star_softc *sc;
	struct star_attach_args *sa;

	sc = device_private(parent);
	sa = aux;

	if (strcmp(cf->cf_name, "starclk") != 0)
		return 0;

	sa->sa_iot = sc->sc_iot;
	sa->sa_dmat = sc->sc_dmat;
	sa->sa_addr = cf->cf_loc[STARCF_ADDR];
	sa->sa_irq = cf->cf_loc[STARCF_IRQ];
	sa->sa_size = 0;

	if (config_match(parent, cf, aux) > 0)
		config_attach(parent, cf, aux, star_print);

	return 0;
}


/* ARGSUSED */
static int
star_search(device_t parent, struct cfdata *cf, const int *ldesc, void *aux)
{
	struct star_softc *sc;
	struct star_attach_args *sa;

	sc = device_private(parent);
	sa = aux;

	if (strcmp(cf->cf_name, "com") == 0) {
		/* 32bit mapped 8bit I/O devices */
		sa->sa_iot = sc->sc_a4x_iot;
	} else {
		/* normal memory mapped device */
		sa->sa_iot = sc->sc_iot;
	}
	sa->sa_dmat = sc->sc_dmat;
	sa->sa_addr = cf->cf_loc[STARCF_ADDR];
	sa->sa_irq = cf->cf_loc[STARCF_IRQ];
	sa->sa_size = 0;

	if (config_match(parent, cf, aux) > 0)
		config_attach(parent, cf, aux, star_print);

	return 0;
}

/* ARGSUSED */
static int
star_print(void *aux, const char *name)
{
	struct star_attach_args *sa;

	sa = (struct star_attach_args *)aux;

	if (sa->sa_size)
		aprint_normal(" addr 0x%lx", sa->sa_addr);
	if (sa->sa_size > 1)
		aprint_normal("-0x%lx", sa->sa_addr + sa->sa_size - 1);
	if (sa->sa_irq >= 0)
		aprint_normal(" irq %d", sa->sa_irq);

	return UNCONF;
}

